/**********************************************************************
 * CD Jukebox by Nathan Strong (gblues@jps.net)
 *
 * This software is covered by the GNU Public License. See COPYING
 * file for details on the license. All code contained in these files
 * is (c) 1999 Nathan Strong.
 **********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <linux/types.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <linux/cdrom.h>
#include <termios.h>
#include <errno.h>

#define CONFIG_FILE "/.cdplayerrc"
#define TRUE  1
#define FALSE 0

typedef unsigned char 		bool;
typedef struct drive_data	DRIVE_DATA;
typedef struct cdrom_tochdr	CD_TOCHDR;
typedef struct option_data	OPTION_DATA;
typedef struct cdrom_tocentry	CD_TOCENTRY;
typedef struct trackinfo	CD_TRACK;
typedef bool OPT_FUN(void);

void load_drives(void);
bool detect_drives(void);
void create_config(void);
void write_config(void);
void load_tochdrs(void);
void boot_ui(void);
void program_loop(void);
void move_cursor_left(void);
void move_cursor_right(void);
void update_ui(void);
void exit_cdplayer(void);
#ifdef DEBUG
void setcorelimit(void);
#endif
void play_track(CD_TRACK *track, int offset);
void stop_playing(void);
void do_unpause(void);

bool do_next_track(void);
bool do_previous_track(void);
bool do_fast_forward(void);
bool do_rewind(void);
bool do_play(void);
bool do_stop(void);
bool do_pause(void);

CD_TRACK *generate_playlist(bool random);

CD_TRACK *new_track(void);
void free_track(CD_TRACK *track);

struct trackinfo
{
  CD_TRACK *	next;
  CD_TRACK *	next_playlist;
  CD_TRACK *	prev;
  CD_TRACK *	prev_playlist;
  DRIVE_DATA *	drive;
  int		length;
  int		start;
  int		frame;
  int		track;
  int		volume;
  int		section;
  long		flags;
};

struct config_data
{
  DRIVE_DATA *drives;
};

struct drive_data
{
  DRIVE_DATA *next;
  char *filename;
  CD_TOCHDR *toc;
  CD_TRACK  *header;
  CD_TRACK  *tracks;
};

struct ui_data
{
  int 		cur_min;
  int 		cur_sec;
  int 		time_format;
  long 		options;
  int 		num_tracks;
  bool		playing;
  long		dirty;
  DRIVE_DATA *	cur_cd;
  CD_TRACK *	cur_track;
  CD_TRACK *	playlist;
  OPTION_DATA *	commands;
};

struct option_data
{
  bool selected;
  OPT_FUN *fun;
  char *name;
};

extern struct config_data	config;
extern struct ui_data		ui;

#define COUNT_UP_TRACK 0

#define IS_SET(bit, flag) ((bit) & (flag))
#define SET_BIT(bit, flag) ( (bit) |= (flag) )

#define DIRTY_CLOCK 0x00000001
#define DIRTY_TRACK 0x00000002
#define DIRTY_UI    0x00000004

extern long current_time;


#define PLAYING 1
#define PAUSED  2
